@@ -1,32 +1,46 @@
+import asyncio
 import logging
 import time
+from asyncio.futures import Future
 from datetime import datetime
 from threading import Thread
 from typing import Optional
 import websocket
 
-from .rustplus_pb2 import AppMessage, AppRequest
+from .rustplus_proto import AppMessage, AppRequest
 from ..structures import RustChatMessage
 from ...exceptions import ClientNotConnectedError
+from ...conversation import ConversationFactory, Conversation, ConversationPrompt
+
+CONNECTED = 1
+PENDING_CONNECTION = 2
+CLOSING = 4
+CLOSED = 3
 
 
 class RustWebsocket(websocket.WebSocket):
-    def __init__(self, ip, port, remote, use_proxy):
+    def __init__(self, ip, port, remote, use_proxy, magic_value):
 
         self.ip = ip
         self.port = port
         self.thread: Thread = None
-        self.open = False
+        self.connection_status = CLOSED
         self.use_proxy = use_proxy
         self.remote = remote
         self.logger = logging.getLogger("rustplus.py")
         self.connected_time = time.time()
+        self.magic_value = magic_value
+        self.outgoing_conversation_messages = []
 
         super().__init__(enable_multithread=True)
 
-    def connect(self, retries = float("inf"), ignore=False) -> None:
+    def connect(
+        self, retries=float("inf"), ignore_open_value: bool = False, delay: int = 20
+    ) -> None:
 
-        if ((not self.open) or ignore) and not self.remote.is_pending:
+        if (
+            not self.connection_status == CONNECTED or ignore_open_value
+        ) and not self.remote.is_pending():
 
             attempts = 0
 
@@ -35,13 +49,13 @@ class RustWebsocket(websocket.WebSocket):
                 if attempts >= retries:
                     raise ConnectionAbortedError("Reached Retry Limit")
 
-                self.remote.is_pending = True
+                self.connection_status = PENDING_CONNECTION
 
                 try:
                     address = (
-                        f"wss://companion-rust.facepunch.com/game/{self.ip}/{self.port}"
+                        f"wss://companion-rust.facepunch.com/game/{self.ip}/{self.port}?v={str(self.magic_value)}"
                         if self.use_proxy
-                        else f"ws://{self.ip}:{self.port} "
+                        else f"ws://{self.ip}:{self.port}?v={str(self.magic_value)}"
                     )
                     super().connect(address)
                     self.connected_time = time.time()
@@ -49,14 +63,14 @@ class RustWebsocket(websocket.WebSocket):
                 except Exception:
                     self.logger.warning(
                         f"{datetime.now().strftime('%d/%m/%Y %H:%M:%S')} "
-                        "[RustPlus.py] Cannot Connect to server. Retrying in 20 seconds"
+                        f"[RustPlus.py] Cannot Connect to server. Retrying in {str(delay)} seconds"
                     )
                     attempts += 1
-                    time.sleep(20)
+                    time.sleep(delay)
 
-            self.remote.is_pending = False
+            self.connection_status = CONNECTED
 
-            self.open = True
+        if not ignore_open_value:
 
             self.thread = Thread(
                 target=self.run, name="[RustPlus.py] WebsocketThread", daemon=True
@@ -65,23 +79,29 @@ class RustWebsocket(websocket.WebSocket):
 
     def close(self) -> None:
 
+        self.connection_status = CLOSING
         super().close()
-        self.open = False
+        self.connection_status = CLOSED
 
-    def send_message(self, message: AppRequest) -> None:
+    async def send_message(self, message: AppRequest) -> None:
         """
         Send the Protobuf to the server
         """
+
+        if self.connection_status == CLOSED:
+            raise ClientNotConnectedError("Not Connected")
+
         try:
-            self.remote.pending_requests[message.seq] = message
             self.send_binary(message.SerializeToString())
+            self.remote.pending_for_response[message.seq] = message
         except Exception:
-            if not self.open:
-                raise ClientNotConnectedError("Not Connected")
+            while self.remote.is_pending():
+                await asyncio.sleep(0.5)
+            return await self.remote.send_message(message)
 
     def run(self) -> None:
 
-        while self.open:
+        while self.connection_status == CONNECTED:
             try:
                 data = self.recv()
 
@@ -91,16 +111,16 @@ class RustWebsocket(websocket.WebSocket):
                 app_message.ParseFromString(data)
 
             except Exception:
-                if self.open:
+                if self.connection_status == CONNECTED:
                     self.logger.warning(
                         f"{datetime.now().strftime('%d/%m/%Y %H:%M:%S')} [RustPlus.py] Connection interrupted, Retrying"
                     )
-                    self.connect(ignore=True)
+                    self.connect(ignore_open_value=True)
                     continue
                 return
 
             try:
-                del self.remote.pending_requests[app_message.response.seq]
+                del self.remote.pending_for_response[app_message.response.seq]
             except KeyError:
                 pass
 
@@ -129,6 +149,31 @@ class RustWebsocket(websocket.WebSocket):
             self.remote.event_handler.run_team_event(app_message)
 
         elif self.is_message(app_message):
+
+            steam_id = int(app_message.broadcast.teamMessage.message.steamId)
+            message = str(app_message.broadcast.teamMessage.message.message)
+
+            if self.remote.conversation_factory.has_conversation(steam_id):
+                if message not in self.outgoing_conversation_messages:
+                    conversation: Conversation = self.remote.conversation_factory.get_conversation(steam_id)
+
+                    conversation.get_answers().append(message)
+                    conversation.run_coro(conversation.get_current_prompt().on_response, args=[message])
+
+                    if conversation.has_next():
+                        conversation.increment_prompt()
+                        prompt = conversation.get_current_prompt()
+                        prompt_string = conversation.run_coro(prompt.prompt, args=[])
+                        conversation.run_coro(conversation.send_prompt, args=[prompt_string])
+                    else:
+                        prompt = conversation.get_current_prompt()
+                        prompt_string = conversation.run_coro(prompt.on_finish, args=[])
+                        if prompt_string != "":
+                            conversation.run_coro(conversation.send_prompt, args=[prompt_string])
+                        self.remote.conversation_factory.abort_conversation(steam_id)
+                else:
+                    self.outgoing_conversation_messages.remove(message)
+
             self.remote.event_handler.run_chat_event(app_message)
 
         else:
@@ -166,7 +211,7 @@ class RustWebsocket(websocket.WebSocket):
         await self.send_message(app_request)
 
     @staticmethod
-    def _get_proto_cost(app_request) -> int:
+    def get_proto_cost(app_request) -> int:
         """
         Gets the cost of an AppRequest
         """
